# Some custom exceptions being used for the custom datatype in the databases package

class InvalidDataType(Exception):

    def __init__(self, message: str):
        self.message = message

    def __str__(self):
        if self.message:
            return self.message

class OperatorNotSupported(Exception):

    def __init__(self, message: str):
        self.message = message

    def __str__(self):
        if self.message:
            return self.message





# This module contains a custom datatype User, supporting most python operators and other functionalities.

# Python imports
import numbers

# Worldnames imports
from worldnames.exceptions import InvalidDataType, OperatorNotSupported

class User:

    """
    This class is a custom data type for fictional users being populated using the worldnames module.
    Supporting arithmetic and logical operators as well as object comparisons.
    """

    def __init__(self, _first_name: str, _last_name: str, gender: str, age: int, email: str):
        """
        The init needs different args to initialize the class. Type checking is not implemented.
        :param _first_name:
        :param _last_name:
        :param age:
        :param gender:
        :param email:
        """
        self.users = set()
        self.first_name = _first_name
        self.last_name = _last_name
        self.gender = gender
        self.age = age
        self.email = email
        self.users.add(self)

    def __iter__(self) -> iter:
        """
        Supporting iterating for example when unpacking.
        :return: iterator
        """
        return iter((self.first_name, self.last_name, self.gender, self.age, self.email))

    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        return f"Name: {self.first_name} {self.last_name} | gender: {self.gender} | age: {self.age} | email: {self.email}"

    # Arithmetic operators
    def __add__(self, other) -> set:
        """
        Supports adding for type User
        :param other:
        :return: set
        """
        if not isinstance(other, User):
            raise InvalidDataType(f"{other} should be an instance of User not {type(other)}")
        else:
            self.users.add(other)
            return self.users

    def __sub__(self, other) -> set:
        """
        Supports subtraction by User.
        :param other:
        :return: set
        """
        if not isinstance(other, User):
            raise InvalidDataType(f"{other} should be an instance of User not {type(other)}")
        else:
            self.users.remove(other)
            return self.users

    def __mul__(self, other) -> list:
        """
        Supports multiplication by number or User.
        :param other:
        :return: list
        """
        if not isinstance(other, User) and not isinstance(other, numbers.Real):
            raise InvalidDataType(f"{other} should be an instance of User or Number not {type(other)}")
        else:
            return list(set) * other.age if isinstance(other, User) else other

    def __truediv__(self, other) -> OperatorNotSupported:
        """
        Does not support true division
        :param other:
        :return: raises an exception
        """
        raise OperatorNotSupported("Division is not supported for User")

    def __floordiv__(self, other) -> OperatorNotSupported:
        """
        Does not support floor division
        :param other:
        :return: raises an exception
        """
        raise OperatorNotSupported("Floor division is not supported for User")

    def __mod__(self, other) -> OperatorNotSupported:
        """
        Does not support mod
        :param other:
        :return: raises an exception
        """
        raise OperatorNotSupported("Modulus is not supported for User")

    def __pow__(self, *args, **kwargs) -> set:
        """
        Powering a User is not supported
        :param power:
        :param modulo:
        :return: raises an exception
        """
        raise OperatorNotSupported("The power of User is not supported")

    # Rich comparisons
    def __eq__(self, other) -> bool:
        """
        Supports comparison for integers and user type
        :param other:
        :return: bool
        """
        if not isinstance(other, User):
            raise InvalidDataType(f"{other} should be an instance of User not {type(other)}")
        else:
            return hash(self) == hash(other)

    def __lt__(self, other) -> bool:
        """
        Supports comparison for integers and user type
        :param other:
        :return: bool
        """
        if not isinstance(other, User) and not isinstance(other, numbers.Real):
            raise InvalidDataType(f"{other} should be an instance of User or Number not {type(other)}")
        else:
            return bool(self.age < other.age if isinstance(other, User) else other)

    def __le__(self, other) -> bool:
        """
        Supports comparison for integers and user type
        :param other:
        :return: bool
        """
        if not isinstance(other, User) and not isinstance(other, numbers.Real):
            raise InvalidDataType(f"{other} should be an instance of User or Number not {type(other)}")
        else:
            return bool(self.age <= other.age if isinstance(other, User) else other)

    def __ge__(self, other) -> bool:
        """
        Supports comparison for integers and user type
        :param other:
        :return: bool
        """
        if not isinstance(other, User) and not isinstance(other, numbers.Real):
            raise InvalidDataType(f"{other} should be an instance of User or Number not {type(other)}")
        else:
            return bool(self.age >= other.age if isinstance(other, User) else other)

    def __gt__(self, other) -> bool:
        """
        Supports comparison for integers and user type
        :param other:
        :return: bool
        """
        if not isinstance(other, User) and not isinstance(other, numbers.Real):
            raise InvalidDataType(f"{other} should be an instance of User or Number not {type(other)}")
        else:
            return bool(self.age > other.age if isinstance(other, User) else other)

    def __hash__(self):
        return hash((self.first_name, self.last_name, self.gender, self.age, self.email))


# The init module for the package worldnames creates an instance of the class WorldNames and assigns
# the bounded methods to variables so they can be accessed while still being Polymorphisme (d)
# Making the class reusable without having separate functions and rather using object-oriented programming.

# Python imports
import random, json, site, os

# world-names imports
from worldnames.populatinghelpers import email_domains, letters, genders

class WorldNames:

    """
    This class holds static and non-static methods for generating a name, age, gender, email or a combination of them all.
    Most likely to be used for populating a database or running some tests.
    """

    def __init__(self, file: str = 'worldnames/worldnames.json'):
        """
        The initializer needs the file name and the package folder which holds the JSON file.
        :param file:
        """
        site_packages = site.getsitepackages()
        site_packages.append("")
        names_in_json = []
        for sp in site_packages:
            try:
                __file = open(os.path.join(sp, file))
                names_in_json = __file.readlines()[0]
                break
            except FileNotFoundError:
                pass
        if not names_in_json:
            message = (
                    "file worldnames.json not found. Looked in the following directories:\n "
                    + os.path.abspath("worldnames/worldnames.json")
                    + "\n "
                    + "\n".join(site_packages)
            )
            raise BaseException(message)
        self.names = json.loads(names_in_json)
        self.min, self.max = 0, len(self.names)-1
        self.min_gender, self.max_gender = 0, len(genders)-1
        self.min_domain, self.max_domain = 0, len(email_domains)-1

    def full_name(self) -> str:
        """
        Returns a random first and last name
        :return: str
        """
        return f"{self.first_name()} {self.last_name()}"

    def first_name(self) -> str:
        """
        Returns a random first name
        :return: str
        """
        random.shuffle(self.names)
        return self.names[random.randint(self.min, self.max)]

    @staticmethod
    def last_name() -> str:
        """
        returns a random last name
        :return: str
        """
        _max = random.randint(3, 12)
        random.shuffle(letters)
        _last_name = "".join(letters[0:_max])
        return f"{_last_name[0].upper()}{_last_name[1::].lower()}"

    @staticmethod
    def age() -> int:
        """
        Returns a random age between 0 and 120
        :return: int
        """
        return random.randint(0, 120)

    def gender(self) -> str:
        """
        Returns a random gender
        :return: str
        """
        random.shuffle(genders)
        return genders[random.randint(self.min_gender, self.max_gender)]

    def email(self, _first_name:str=None, _last_name:str=None) -> str:
        """
        Returns a random email
        :param _first_name:
        :param _last_name:
        :return: str
        """
        random.shuffle(email_domains)
        domain = email_domains[random.randint(self.min_domain, self.max_domain)]
        if not first_name or not last_name:
            return f"{self.first_name()}.{self.last_name()}@{domain}"
        else:
            return f"{_first_name}.{_last_name}@{domain}"

    def user(self) -> tuple:
        """
        Returns a random user containing the first_name, last_name, age, gender and email in a tuple.
        :return: tuple
        """
        fn, ln = self.first_name(), self.last_name()
        return fn, ln, self.gender(), self.age(), self.email(fn, ln)

world_names = WorldNames()
full_name = world_names.full_name
first_name = world_names.first_name
last_name = world_names.last_name
age = world_names.age
gender = world_names.gender
email = world_names.email
user = world_names.user



# An enumeration for the databases being used in the JSON(CLI) program. Simple enumeration with 2 members.

#Python imports
from enum import Enum

# Worldnames imports
from worldnames.databases.my_sql import MySQL
from worldnames.databases.sqlite import Sqlite

class Databases(Enum):
    sqlite = Sqlite
    mysql = MySQL


# This module contains a MySQL class thats being used to connect to a MySQL database.

# Python imports
import time, os, sys, platform

# 3th part imports
from mysql import connector
from mysql.connector import (errorcode, cursor)
from rich.console import Console
from tabulate import tabulate

# Worldnames imports
from worldnames.content import logo
from worldnames.content import custom_print
from worldnames.databases.sqlsharing import SqlShared

operating_system = platform.system().lower()
clear = "clear" if operating_system == "linux" or operating_system == "darwin" else "cls"
console = Console()


class MySQL(SqlShared):

    """
    A MySQL class to simulate populating a database using MySQL inheriting from SqlShared containing methods to populate the mysql database.
    """

    def __init__(self) -> None:
        self.connected = False
        self.users = list()
        self.databases = None
        self.validators = {
            'user_name': False,
            'password': False,
            'host': False,
            'port': False
        }
        self.connection_attempt = False
        while True:
            os.system(clear)
            console.print(logo)
            custom_print("Om een verbinding te kunnen maken met jouw MySQL database. Dien je de volgende informatie in te vullen: \n")
            if not self.validators['user_name']:
                self.user_name = input("Gebruikersnaam: ").strip()
                self.validators['user_name'] = True
            if not self.validators['password']:
                self.password = input("Wachtwoord: ").strip()
                self.validators['password'] = True
            if not self.validators['host']:
                self.host = input("Host: ").strip()
                self.validators['host'] = True
            if not self.validators['port']:
                self.port = input("Poort: ").strip()
                self.validators['port'] = True
            try:
                custom_print("\nVerbinding maken met MySQL...")
                self.cnx = self.connect()
            except connector.Error as error:
                if error.errno == errorcode.ER_ACCESS_DENIED_ERROR:
                    custom_print("Gebruikersnaam of wachtwoord is ongeldig!")
                    self.validators['user_name'] = False
                    self.validators['password'] = False
                    time.sleep(2)
                    continue
                elif error.errno in [errorcode.CR_CONN_HOST_ERROR, -1, errorcode.CR_UNKNOWN_HOST, errorcode.ER_BAD_HOST_ERROR]:
                    custom_print(f"Host {self.host} of poort {self.port} is ongeldig!")
                    self.validators['port'] = False
                    self.validators['host'] = False
                    time.sleep(2)
                    continue
                else:
                    custom_print("Something went wrong, create a ISSUE on: https://github.com/ayoub-abdessadak/worldnames/issues")
                    custom_print(f"ERROR: {error.errno}, {error.msg}, {error}, {error.sqlstate}")
                    sys.exit()
            else:
                custom_print("Verbinding successvol")
                self.connection_attempt = True
                self.disconnect()
                return

    def connect(self) -> connector:
        """
        class method to connect to the mysql database
        :return: a mysql connector
        """
        self.connected = True
        return connector.connect(user=self.user_name, password=self.password, host=self.host, port=self.port)

    def disconnect(self) -> None:
        """
        class method to disconnect to the mysql database
        :return: None
        """
        self.connected = False
        self.cnx.close()

    def get_cursor(self) -> cursor:
        """
        class method to lazy retrieve a cursor
        :return: a mysql cursor
        """
        if not self.connected:
            self.cnx = self.connect()
        return self.cnx.cursor(buffered=True)

    def run(self) -> None:
        """
        class method to run the populating simulator and start an infinite loop, letting the user search through a table.
        :return: None
        """
        table_name = "Users"
        super().create_table(self.get_cursor(), table_name, False)
        super().fill_table(self.get_cursor(), self.cnx, table_name, 20)
        super().view_users(self.get_cursor(), table_name)
        while True:
            os.system(clear)
            console.print(logo)
            custom_print(self)
            _exit = super().search_user(self.get_cursor(), None, table_name)
            if _exit:
                break

    def __repr__(self) -> str:
        """
        printing the table nicely using tabulate
        :return: string
        """
        return f"""{tabulate(self.users, headers=self.headers, tablefmt="fancy_grid")}\n"""



# This module holds the program written in JSON that's being used in the RunJsonProgram class.
# The goal is to reduce the amount of conditional statements being used to write a CLI program.

# Worldnames imports
from worldnames.databases import Databases

# 3th party imports
from colorama import Fore

program = {
    1: {
        "title": "User Populater WorldNames - Een programma om fictieve gebruikers aan te maken",
        "question": "Klik op enter om door te gaan...",
        "action": "",
        "options": f"",
        "option_values": None,
        "enum": None,
        "next": 2,
        "color":Fore.GREEN,
    },
    2: {
        "title": "Simulatie (in dit programma wordt een simulatie uitgevoerd voor het populeren van een database)\n",
        "question": "\nMaak een keuze: ",
        "action": "call",
        "options": f"1. Simuleren via MySQL\n2. Simuleren via SqlLite",
        "option_values": {
            "1": "mysql",
            "2": "sqlite",
        },
        "enum": Databases,
        "next": 3,
        "color": Fore.GREEN,
    },
    3: {
        "title": "WorldNames Hoofdmenu\n",
        "question": "\nKeuze: ",
        "action": "flow",
        "options": f"1. Om opnieuw een simulatie uit te voeren\n2. Om het programma te verlaten",
        "option_values": {
            "1": 2,
            "2": -1,
        },
        "enum": None,
        "next": 3,
        "color": Fore.GREEN,
    },
}




# This module runs the RunJsonProgram class and specifically the run_program method bound to an instance of that class.
# That method contains some conditional statements that causing the program to loop through a dictionary. It can be extended for more functionalities.

# Python imports
import time, os, sys, platform

# 3th party imports
from rich.console import Console

# world-names imports
from worldnames.flow import program
from worldnames.content import logo, icon

operating_system = platform.system().lower()
clear = "clear" if operating_system == "linux" or operating_system == "darwin" else "cls"
console = Console()

class RunJsonProgram:
    """
    This class is written to run a CLI program that can be written in JSON (FROM AN PYTHON MODULE).
    """
    def __init__(self) -> None:
        self.database = None
        self.showed_icon = False

    def run_program(self, program_flow: dict) -> None:
        """
        The run program method bound to the created instance of RunJsonProgram runs a program that can be written in JSON.
        See the program in flow.py
        :param program_flow: Program in JSON
        :return: None
        """
        _next = 1
        while True:
            if _next == -1:
                print("Tot ziens :)")
                sys.exit()

            flow = program_flow[_next]
            os.system(clear)
            console.print(self.__logo())
            print(flow["color"] + f'''\n{flow["title"]}\n{flow["options"]}''')
            choice = input(flow["question"])

            if flow["action"] == "call":
                try:
                    output = flow['enum'][flow['option_values'][choice.strip()]].value()
                    output.run()
                    _next = flow["next"]
                    continue
                except KeyError:
                    print("De opgegeven keuze is ongeldig.")
                    time.sleep(2)
                    continue
                except Exception as error:
                    print(f"Something went wrong, make a issue on GIT, for {error}")
                    sys.exit()

            if flow["action"] == "flow":
                try:
                    _next = flow["option_values"][choice.strip()]
                    continue
                except KeyError:
                    print("De opgegeven keuze is ongeldig.")
                    time.sleep(2)
                    continue
                except Exception as error:
                    print(f"Something went wrong, make a issue on GIT, for {error}")
                    sys.exit()

            if not flow["action"]:
                _next = flow["next"]
                continue

    def __logo(self) -> str:
        """
        If you call me I will return once an icon and always the worldnames logo. You should print me.
        :return: str
        """
        if not self.showed_icon:
            self.showed_icon = True
            return logo + icon
        else:
            return logo


if __name__ == "__main__":
    rjp = RunJsonProgram()
    rjp.run_program(program)



# This module contains a Sqlite class that's being used to create or connect to a sqlite database.

# Python imports
from datetime import datetime
import sqlite3, os, sys, platform

# 3th party imports
from tabulate import tabulate
from rich.console import Console

# Worldnames imports
from worldnames.databases.sqlsharing import SqlShared
from worldnames.content import logo
from worldnames.content import custom_print

operating_system = platform.system().lower()
clear = "clear" if operating_system == "linux" or operating_system == "darwin" else "cls"
console = Console()
console.print(logo)

class Sqlite(SqlShared):

    """
    Class for sim-populating a sqlite database.
    """

    def __init__(self, simulation: bool=True) -> None:
        """
        on init creating a test database or opening an existing database.
        :param simulation:
        """
        self.tables = None
        self.table_name = None
        self.users = list()
        if simulation:
            self.database_name = f"SIMDATABASE-{datetime.now().isoformat()}"
            file = open(self.database_name, "w")
            file.close()
        else:
            self.database_name = input("Geef de sqlite.db file op met het volledige pad naar het bestand: ")
        try:
            self.con = sqlite3.connect(self.database_name)
            self.cursor = self.con.cursor()
        except Exception as error:
            custom_print(f"Create a issue for error {error} on github")
            sys.exit()

    def run(self) -> None:
        """
        class method to run the populating simulator and start an infinite loop, letting the user search through a table.
        :return: None
        """
        table_name = "Users"
        super().create_table(self.cursor, table_name, True)
        super().fill_table(self.cursor, self.con, table_name, 20)
        super().view_users(self.cursor, table_name)
        while True:
            os.system(clear)
            console.print(logo)
            custom_print(self)
            _exit = super().search_user(self.cursor, None, table_name)
            if _exit:
                break

    def __repr__(self) -> str:
        """
        printing the table nicely using tabulate
        :return: string
        """
        return f"""{tabulate(self.users,headers=self.headers,tablefmt="fancy_grid")}\n"""




# Some data that is being used in the WorldNames module to generate data

# Python imports
import string

email_domains, letters, genders= [
    "gmail.com",
    "yahoo.com",
    "outlook.com",
    "hotmail.com",
    "aol.com",
    "icloud.com",
    "protonmail.com",
    "yandex.com",
    "mail.com",
    "zoho.com"
], [letter for letter in string.ascii_lowercase], ['Agender',
 'Androgyne',
 'Androgynous',
 'Bigender',
 'Cis',
 'Cis',
 'Female',
 'Cis',
 'Male',
 'Cis',
 'Man',
 'Cis',
 'Woman',
 'Cisgender',
 'Cisgender',
 'Female',
 'Cisgender',
 'Male',
 'Cisgender',
 'Man',
 'Cisgender',
 'Woman',
 'Female',
 'Female',
 'to',
 'Male',
 'FTM',
 'Gender',
 'Fluid',
 'Gender',
 'Identity',
 'Gender',
 'Nonconforming',
 'Gender',
 'Questioning',
 'Gender',
 'Variant',
 'Genderqueer',
 'Intersex',
 'Male',
 'Male',
 'to',
 'Female',
 'MTF',
 'Neither',
 'Neutrois',
 'Non-Binary',
 'Non-binary',
 'Not',
 'specified',
 'Other',
 'Pangender',
 'Trans',
 'Trans',
 'Female',
 'Trans',
 'Male',
 'Trans',
 'Man',
 'Trans',
 'Person',
 'Trans',
 'Woman',
 'Trans*',
 'Trans*',
 'Female',
 'Trans*',
 'Male',
 'Trans*',
 'Man',
 'Trans*',
 'Person',
 'Trans*',
 'Woman',
 'Transfeminine',
 'Transgender',
 'Transgender',
 'Female',
 'Transgender',
 'Male',
 'Transgender',
 'Man',
 'Transgender',
 'Person',
 'Transgender',
 'Woman',
 'Transmasculine',
 'Transsexual',
 'Transsexual',
 'Female',
 'Transsexual',
 'Male',
 'Transsexual',
 'Man',
 'Transsexual',
 'Person',
 'Transsexual',
 'Woman',
 'Two-Spirit']


# This module contains a SqlShared class that's being used in both sqlite and mysql module in the databases package.

#Python imports
import os, time, platform
from uuid import uuid4

# 3th party imports
from tabulate import tabulate
from yaspin import yaspin
from mysql.connector import cursor as _cursor
from rich.console import Console

# Worldnames imports
import worldnames
from worldnames.content import logo
from worldnames.content import custom_print
from worldnames.databases.customdatatypes import User

operating_system = platform.system().lower()
clear = "clear" if operating_system == "linux" or operating_system == "darwin" else "cls"
console = Console()
console.print(logo)

class SqlShared:

    """
    Class with methods that are being used when populating a MySQL or sqlite database.
    """

    headers: list = ["Voornaam", "Achternaam", "Gender", "Leeftijd", "Email"]
    users: list = []

    @staticmethod
    def wait(seconds: int | float) -> None:
        """
        When called I display a spinning icon for the given seconds.
        :param seconds:
        :return: None
        """
        with yaspin():
            time.sleep(seconds)

    def create_table(self, cursor:_cursor, table_name: str="Users", lite=True) -> bool:
        """
        This class method is responsible to create a table in SQL, supports MySQL and Sqlite
        :param cursor:
        :param table_name:
        :param lite:
        :return: bool, true if created, false if not
        """
        # Test Database maken for MySQL
        if not lite:
            custom_print("Tijdelijke database maken...")
            db_name = "db" + uuid4().__str__()[0:10].replace("-", '')
            cursor.execute(f"CREATE DATABASE {db_name}")
            cursor.execute(f"USE {db_name};")
            self.wait(1)
            custom_print(f"Database: {db_name} aangemaakt")
            self.wait(0.5)
        # Tabel maken in MySQL of Sqlite test DB
        custom_print(f"Tabel {table_name} maken...")
        query = f"CREATE TABLE {table_name}(first_name CHAR(255), last_name CHAR(255), gender CHAR(255), age INT, email CHAR(255))"
        cursor.execute(query)
        self.wait(1)
        # Tabellen ophalen voor sqlite of mysql
        if lite:
            cursor.execute("SELECT name FROM sqlite_master WHERE type='table'")
        else:
            cursor.execute("SHOW TABLES;")
        tables = [table[0] for table in cursor.fetchall()]
        if table_name in tables:
            custom_print(f"{table_name} is successvol aangemaakt")
            self.wait(1)
            return True
        else:
            custom_print(f"Het is niet gelukt om de tabel {table_name} te maken.")
            self.wait(2)
            return False

    def add_user(self, user: tuple | User, cursor:_cursor, con, table_name: str="Users") -> None:
        """
        This class method is responsible for adding a user. When called user is added into table destined for that DB.
        :param user: tuple or User
        :param cursor: sql cursor
        :param con: sql connector
        :param table_name: str
        :return: none
        """
        if isinstance(user, User):
            user = tuple([*user])
        custom_print(f"Gebruiker: {user} toevoegen")
        query = f"INSERT INTO {table_name} (first_name, last_name, gender, age, email) VALUES {user}"
        cursor.execute(query)
        con.commit()
        self.wait(0.23)

    def fill_table(self, cursor:_cursor, con, table_name: str="Users", amount_of_users: int = 20) -> None:
        """
        Class method to fill a table with users, default is 20 users. Users will be populated using the world-names populater
        :param cursor: sql cursor
        :param con: sql connector
        :param table_name: str
        :param amount_of_users: int
        :return: None
        """
        for _ in range(amount_of_users):
            self.add_user(worldnames.user(), cursor, con, table_name)

    def view_users(self, cursor:_cursor, table_name: str = "Users") -> None:
        """
        Class method to view users that have been populated
        :param cursor: sql cursor
        :param table_name: str
        :return: None
        """
        os.system(clear)
        console.print(logo)
        custom_print(f"Users for table: {table_name}\n")
        query = f"SELECT * from {table_name}"
        cursor.execute(query)
        res = cursor.fetchall()
        users = res
        self.users = [User(*user) for user in users] # Custom data type User
        custom_print(tabulate(self.users, headers=self.headers, tablefmt="fancy_grid")) # The implemented iterator of the custom data type is used by tabulate
        input("\nKlik op enter om verder te gaan...")

    def search_user(self, cursor:_cursor, search_input: str=None, table_name: str = "Users") -> str:
        """
        Class method that let the user search through a table.
        :param cursor: sql cursor
        :param search_input: str
        :param table_name: str
        :return: str (exit)
        """
        custom_print("Typ exit om de zoekfunctie te verlaten\n")
        search_input = input("Zoek: ") if not search_input else search_input
        if search_input.lower().strip() == "exit":
            return "exit"
        fields = ["first_name", "last_name", "gender", "age", "email"]
        search_input = search_input.split()
        results = []
        for field in fields:
            for si in search_input:
                query = f"SELECT * FROM {table_name} WHERE {field} LIKE '%{si}%'"
                cursor.execute(query)
                result = cursor.fetchall()
                results = [*results, *result]
        results = set(results)
        self.wait(0.1)
        os.system(clear)
        console.print(logo)
        custom_print(f"Gevonden resultaten voor zoekopdracht: {' '.join(search_input)}")
        custom_print(tabulate(results,headers=self.headers,tablefmt="fancy_grid"))
        input("\nEnter om verder te gaan...")